// device file for the Freiburger Stepping Motor
// device related parameter(s): 'TLAM' (timeout) and CAMAC-Address 'N'
// device specific parameter(s): moving direction 'direct', 'stepsize' and 'position' and 'shots'
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// Attention: Difference between VME based systems and Linux:
//            Command completion character is '\x0d' (ASCII #13)
//            ==> VME:   '\x0d' = '\n' (Linux: <CR>+<LF>)
//            ==> Linux: '\x0d' = '\r'
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// M.Dammasch, Jan-Apr/98
//
//M. Dammasch 05.11.1998 upgrade to v2.0beta
//
// last edit: 1998/11/05
//

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include "list.hh"
#include "sched.hh"
#include "camacs.h"
#include "error.hh"

#define  CAMMOT_STR_MAX  80
#define  MOTOR_BUSY      1600

// device variable(s)
static int N, TLAM, direct, stepsize, position, shots;

// cammot(): convert verbose command string (ASCII)
//           into unsigned integer(s) for CAMAC command(s) 
static void cammot ( int f, int a, int n, char *string )
{
  int i;
  uint c;

  for (i=0; string[i] != '\0'; i++)
  {
    c = (uint) string[i];
    printf("\nfr_stepper: '%c' (%d) [0x%x] <%d%d%d%d%d%d%d%d>",string[i],c,c,(c&128?1:0),(c& 64?1:0),(c& 32?1:0),(c& 16?1:0),(c&  8?1:0),(c&  4?1:0),(c&  2?1:0),(c&  1?1:0));
    camac(f,a,n,&c);
    usleep(1);  // timing problem of the self-brewed stepper driver
  }
}

// init function: called each time before measurement starts
int fr_stepper_init(List *device_data, Scheduler *parameters)
{
  int  i, A=0;
  char bef[CAMMOT_STR_MAX+1];  //verbose command string
	
  if (camac_init() != 1)
    return parameters->error(E_GPIB,"CAMAC ibinit err");

  printf("\ninitializing fr_stepper...");
  // get (external) values for 'TLAM', CAMAC-Slot-Number 'N',
  // moving direction 'direct' and 'stepsize'
  i = device_data->getvar(&TLAM,"all","timeout"); 
  i = device_data->getvar(&N,"fr_stepper","N");
  i = device_data->getvar(&direct,"fr_stepper","direct");
  i = device_data->getvar(&stepsize,"fr_stepper","stepsize");
  i = device_data->getvar(&position,"fr_stepper","position");
  i = device_data->getvar(&shots,"fr_stepper","shots");

  camac(25,8,N,NULL);

  // initialize stepping motor
  printf(" done \nstarting fr_stepper...");
  strcpy(bef,"I\r\0");
  cammot(16,A,N,bef);
  strcpy(bef,"S 2\r\0");
  cammot(16,A,N,bef);
  strcpy(bef,"R 50\r\0");
  cammot(16,A,N,bef);
  strcpy(bef,"F 1\r\0");
  cammot(16,A,N,bef);
  strcpy(bef,"A\r\0");
  cammot(16,A,N,bef);
  printf(" done ");

  return 1;
}

int fr_stepper_pre(List *device_data, Scheduler *parameters)
{
  int  A=0, i=0, result;
  uint bus;
  char bef[CAMMOT_STR_MAX+1];  //verbose command string
  int  inc;  // increment for stepmotor position
  
  //printf("\nstepper(%d/%d,%d/%d)",parameters->act_channel(),parameters->maximal_channels(),parameters->act_integrations(),parameters->maximal_integrations());
  if ( ! (parameters->act_channel() % shots == 0 && parameters->act_integrations() == 0)) return 1;

  if (direct == 1)
  { 
    // forward direction
    strcpy(bef,"+\r\0");
    cammot(16,A,N,bef);
    inc = 1;
  }
  else
  {
    // reverse direction
    strcpy(bef,"-\r\0");
    cammot(16,A,N,bef);
    inc = -1;
  }

  // set 'stepsize'
  sprintf(bef,"N %d\r",stepsize);
  cammot(16,A,N,bef);
  position += inc*stepsize;

  // Command: GO!
  strcpy(bef,"G\r\0");
  cammot(16,A,N,bef);

  // get busy value
  camac(1,A,N,&bus);

  // wait until not busy anymore
  while(( ++i <( TLAM-1)) && (bus>MOTOR_BUSY))
    camac(1,A,N,&bus);
	
  if ( i >= TLAM )
  {
    result=parameters->error(E_TIMEOUT,"fr_stepper timed out ");
    return result;   //return FALSE -> measurement stops
  }

  // wait until final position is reached: move again 0 (zero) steps further
  strcpy(bef,"N 0\r\0");
  cammot(16,A,N,bef);
  strcpy(bef,"G\r\0");
  cammot(16,A,N,bef);
  camac(1,A,N,&bus);
  while(bus > MOTOR_BUSY)
  {
#ifdef DEBUG
    printf("\nfr_stepper: still moving ");
#endif
    usleep(5*stepsize);
    camac(1,A,N,&bus);
  }	

#ifdef DEBUG
  printf("\nfr_stepper: completed ");
#endif

  return 1;
}

int fr_stepper(List *device_data, Scheduler *parameters)
{
  // no operation

  return 1;
}


int fr_stepper_close(List *device_data, Scheduler *parameters)
{

  printf("\nclosing fr_stepper ");
  device_data->putvar(position,"fr_stepper","position");
  if (camac_close() != 1)
    return parameters->error(E_GPIB,"CAMAC ibclose err");

  return 1; //function has to return a value -> >1 otherwise measurement stops
}






































