//device file for Stanford Delay/Pulse Generator Model DG535
//M. Dammasch 28.07.1998

//M. Dammasch 05.11.1998 upgrade to v2.0beta
//M. Dammasch 15.12.1998 error()-handling changed (only 0 returns)

//this header has always to be included !!
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
//#include <ib.h>
#include <gpiberr.h>
#include <gpib.h>
#include "list.hh"
#include "sched.hh"
#include "error.hh"

/**
   integer codes for the delay and output commands
**/
#define  IN_TRIG  0   // trigger input    
#define  OUT_T0   1   // T0 output
#define  OUT_A    2   // A output
#define  OUT_B    3   // B output
#define  OUT_AB   4   // AB & -AB outputs
#define  OUT_C    5   // C output
#define  OUT_D    6   // D output
#define  OUT_CD   7   // CD & -CD outputs

#define  IMP_50  0   // termination impedance 50 ohm
#define  IMP_HZ  1   // termination impedance high-Z

#define  MODE_TTL  0   // output mode TTL
#define  MODE_NIM  1   // output mode NIM
#define  MODE_ECL  2   // output mode ECL
#define  MODE_VAR  3   // output mode VARiable

#define  POL_INV   0  // output polarity inverted
#define  POL_NORM  1  // output polarity normal

#define  TRG_INT  0  // trigger mode internal
#define  TRG_EXT  0  // trigger mode external
#define  TRG_SS   0  // trigger mode single shot
#define  TRG_BUR  0  // trigger mode burst

#define  SLP_FALL  0  // trigger slope on falling edge
#define  SLP_RISE  1  // trigger slope on rising edge

/**
   global variables
**/
static GPIB_Device  srs;                         // Gpib-device structure 
static double       del_a, del_b, del_c, del_d;  // delay time of A, B, C, D relative to T0 (!)
static double       dstep_a, dstep_b, dstep_c, dstep_d;  // delay stepps for A, B, C, D to be added for every channel
static char         errmsg[51];  // local error message string
static char         qterrmsg[81]; // global error message string for qt-error()-widget
static char        *strch[] = {"trigger input",
			       "output T0",
			       "output A",
			       "output B",
			       "output AB",
			       "output C",
			       "output D",
			       "output CD",
};

/*****************************************************************************/
static int set_delay ( GPIB_Device *dev, int ch, double del )
{
  if ( !gpib_write(dev,"DT %d,%d,%G",ch,OUT_T0,del))
  {
    sprintf(errmsg,"Couldn't set delay (%G) for %s ",del,strch[ch]);
    return 0;
  }
  return 1;
}
static int get_delay ( GPIB_Device *dev, int ch, double *del )
{
  char delay[GPIB_STRMAX+1];

  if ( !gpib_write(dev,"DT %d",ch))
  {
    sprintf(errmsg,"Couldn't send 'DT %d'-cmd for %s",ch,strch[ch]);
    return 0;
  }
  if ( !gpib_read(dev,delay,GPIB_STRMAX))
  {
    sprintf(errmsg,"Couldn't get delay for %s",strch[ch]);
    return 0;
  }
  *del = atof(delay+2);

  return 1;
}
/*****************************************************************************/

/**
   the device functions for SRS_DG535, called srs_delay
**/
int srs_delay_init(List *device_data, Scheduler *parameters)
{
  //init Stanford Function Generator
  srs = gpib_init("srs_dg535",IB_REM|IB_IFC);
  if (srs.stat != GPIB_OK)
  {
    if ( !parameters->error(E_GPIB,"GPIB init error "))
      return 0;
  }

  if ( !gpib_write(&srs,"DS Running_under_MAX..."))
  {
    if ( !parameters->error(E_GPIB,"GPIB write error "))
      return 0;
  }
  sleep(1);
  if ( !gpib_write(&srs,"DS"))
  {
    if ( !parameters->error(E_GPIB,"GPIB write error "))
      return 0;
  }
    
  device_data->getvar(&del_a,"srs_delay","delay_A");
  device_data->getvar(&del_b,"srs_delay","delay_B");
  device_data->getvar(&del_c,"srs_delay","delay_C");
  device_data->getvar(&del_d,"srs_delay","delay_D");

  device_data->getvar(&dstep_a,"srs_delay","dstep_A");
  device_data->getvar(&dstep_b,"srs_delay","dstep_B");
  device_data->getvar(&dstep_c,"srs_delay","dstep_C");
  device_data->getvar(&dstep_d,"srs_delay","dstep_D");

  printf("\ninitializing srs_delay");

  if (del_a > 0.0)
  {
    if ( !set_delay(&srs,OUT_A,del_a))
    {
      sprintf(qterrmsg,"SRS_DG535: SET_DELAY(): Error due to\n%s ",errmsg);
      if ( !parameters->error(E_GPIB,qterrmsg))
	return 0;
    }
  }
  if (del_b > 0.0)
  {
    if ( !set_delay(&srs,OUT_B,del_b))
    {
      sprintf(qterrmsg,"SRS_DG535: SET_DELAY(): Error due to\n%s ",errmsg);
      if ( !parameters->error(E_GPIB,qterrmsg))
	return 0;
    }
  }
  if (del_c > 0.0)
  {
    if ( !set_delay(&srs,OUT_C,del_c))
    {
      sprintf(qterrmsg,"SRS_DG535: SET_DELAY(): Error due to\n%s ",errmsg);
      if ( !parameters->error(E_GPIB,qterrmsg))
	return 0;
    }
  }
  if (del_d > 0.0)
  {
    if ( !set_delay(&srs,OUT_D,del_d))
    {
      sprintf(qterrmsg,"SRS_DG535: SET_DELAY(): Error due to\n%s ",errmsg);
      if ( !parameters->error(E_GPIB,qterrmsg))
	return 0;
    }
  }
    
  return 1;
}

int srs_delay_close(List *device_data, Scheduler *parameters)
{
  printf("\nclosing srs_delay ");
  return 1;
}

//pre-trigger device function for Stanford Delay/Pulse Generator
int srs_delay_pre(List *device_data, Scheduler *parameters) 
{
  // increase|decrease delays

  if (dstep_a > 0.0)
  {
    del_a += dstep_a;

    if ( !set_delay(&srs,OUT_A,del_a))
    {
      sprintf(qterrmsg,"SRS_DG535: SET_DELAY(): Error due to\n%s ",errmsg);
      if ( !parameters->error(E_GPIB,qterrmsg))
	return 0;
    }
  }
  if (del_b > 0.0)
  {
    del_b += dstep_b;

    if ( !set_delay(&srs,OUT_B,del_b))
    {
      sprintf(qterrmsg,"SRS_DG535: SET_DELAY(): Error due to\n%s ",errmsg);
      if ( !parameters->error(E_GPIB,qterrmsg))
	return 0;
    }
  }
  if (del_c > 0.0)
  {
    del_c += dstep_c;

    if ( !set_delay(&srs,OUT_C,del_c))
    {
      sprintf(qterrmsg,"SRS_DG535: SET_DELAY(): Error due to\n%s ",errmsg);
      if ( !parameters->error(E_GPIB,qterrmsg))
	return 0;
    }
  }
  if (del_d > 0.0)
  {
    del_d += dstep_d;

    if ( !set_delay(&srs,OUT_D,del_d))
    {
      sprintf(qterrmsg,"SRS_DG535: SET_DELAY(): Error due to\n%s ",errmsg);
      if ( !parameters->error(E_GPIB,qterrmsg))
	return 0;
    }
  }

  return 1;
}

//main device function for Stanford Delay/Pulse Generator
int srs_delay(List *device_data, Scheduler *parameters) 
{
  // no operation in channels|integrations

  return 1; 
}

/* of  Stanford Delay/Pulse Generator */













